##################################################################################
##################################################################################
# SEM-4_EXERCISE-1: STRUCTURAL EQUATION MIXTURE MODELING
# TOPICS: DIRECT AND INDIRECT APPLICATIONS IN R/NLSEM AND MPLUS 
#
# THIS SYNTAX INCLUDES ALL ANALYSES FOR THE EXERCISES
# EXTERNAL FILES:
##################################################################################
##################################################################################

##################################################
##################################################
# EXERCISE 1. growth curve mixture modeling
# A direct application with CD4 cell counts
##################################################
##################################################
library(plotsemm)
library(nlsem)

##################################################
##################################################
# EXERCISE 2: SEMM with indirect application
##################################################
##################################################
# setwd("C:/holger/Workshops/nonlinear/exercises/data")
dat <- read.table("pisa_sem.dat",header=T)

model.semm1 <- '
class: 1
xi1  =~ 1*x1+1*x2
xi2  =~ 1*x3+1*x4
eta  =~ 1*y1+1*y2
eta ~ xi1+xi2
class: 2
xi1  =~ 1*x1+1*x2
xi2  =~ 1*x3+1*x4
eta  =~ 1*y1+1*y2
eta ~ xi1+xi2
'

model.semm1a <- lav2nlsem(model.semm1, constraints = "indirect", class.spec = "class")
model.semm1a$matrices
count_free_parameters(model.semm1a)

# equivalent
model.semm1a <- specify_sem(num.x=4, num.y=2, num.xi=2, num.eta=1, xi="x1-x2,x3-x4",
                            eta="y1-y2",num.classes=2,constraints = "indirect")
model.semm1a$matrices

#####################
specs <- as.data.frame(model.semm1a)
specs[specs$label %in% paste0("Lambda.x", c(2,8)), c("class1","class2")] <- 1
specs[specs$label %in% paste0("Lambda.y", 2), c("class1","class2")] <- 1
specs[specs$label %in% paste0("nu.x", 1:4), c("class1","class2")] <- 0
specs[specs$label %in% paste0("nu.y", 2), c("class1","class2")] <- 0
model.semm1a <- create_sem(specs)
model.semm1a$matrices

count_free_parameters(model.semm1a)

for (i in 1:100) {
  start1 <- runif(count_free_parameters(model.semm1a))
  
  tryCatch({
    semm1 <- em(model.semm1a, dat, start1, verbose=T, convergence=.1)
    #nsemm.ind <- em(model.ind, dat, start.ind, verbose=T, convergence=1)
  }, error=function(e) print(e))
  
  print(i)
  if (length(ls(pattern="semm1")) != 0) break
  
}
# => The model does not converge



##################################################
##################################################
# EXERCISE 3: NSEMM with indirect application
##################################################
##################################################

dat <- read.table("pisa_online1.dat",header=T,na.strings = "999")
#delete missings: nlsem does not allow missing at the moment
dat <- na.omit(dat)


model.ind <- specify_sem(num.x=6, num.y=3, num.xi=2, num.eta=1,
                         xi="x1-x3,x4-x6", eta="y1-y3", num.classes=2, constraints="indirect",
                         interaction="xi1:xi1,xi1:xi2,xi2:xi2")


for (i in 1:100) {
  
  start.ind <- runif(count_free_parameters(model.ind))
  
  tryCatch({
    nsemm.ind <- em(model.ind, dat, start.ind, verbose=T, convergence=1)
  }, error=function(e) print(e))
  
  print(i)
  if (length(ls(pattern="nsemm.ind")) != 0) break
  
}




















