##################################################################################
##################################################################################
# SEM-4_EXERCISE-2: NONLINEAR STRUCTURAL EQUATION MIXTURE MODELING
# TOPICS: DIRECT AND INDIRECT APPLICATIONS IN R/NLSEM AND MPLUS 
#
# THIS SYNTAX INCLUDES ALL ANALYSES FOR THE EXERCISES
# EXTERNAL FILES:
# lms_mplus.inp, lms_mplus_with_standardization.inp
# nsemm2_pisa.inp
# phimix_fun.r: STANDARDIZATION
##################################################################################
##################################################################################
library(MplusAutomation)

##################################################
##################################################
# EXERCISE 3. NSEMM
# An indirect application of a nonlinear SEMM
##################################################
##################################################
setwd("C:/holger/Workshops/nonlinear/exercises/day2/mplus_version")
res.nsemm <- extractModelParameters("nsemm2_pisa.out")

#check directory
#setwd("C:/holger/Workshops/nonlinear/exercises/day2/online/workshop")
# this includes a function for the covariance matrix of xi
source("r/phimix_fun.r")

##################################################
# standardization of coefficients
##################################################

# latent class mean and probability
muc <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="Means"&
                                  res.nsemm$unstandardized$param=="CW#1",3]
pi1 <- exp(c(muc,0))/sum(exp(c(muc,0)))

#means xi
kap1 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="Means"&
                                   res.nsemm$unstandardized$param=="ONL",3]
kap2 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="Means"&
                                   res.nsemm$unstandardized$param=="ATT",3]

# (co)variances xi
ph11 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="Variances"&
                                   res.nsemm$unstandardized$param=="ONL",3]
ph22 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="Variances"&
                                   res.nsemm$unstandardized$param=="ATT",3]
ph12 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="ATT.WITH"&
                                   res.nsemm$unstandardized$param=="ONL",3]

# intercept
alp1 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="Intercepts"&
                                   res.nsemm$unstandardized$param=="READ"&
                                   res.nsemm$unstandardized$LatentClass==1,3]

# regression coefficients
b1 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="READ.ON"&
                                 res.nsemm$unstandardized$LatentClass==1,3]

# residual variance
ps1 <- res.nsemm$unstandardized[res.nsemm$unstandardized$paramHeader=="Residual.Variances"&
                                  res.nsemm$unstandardized$param=="READ"&
                                  res.nsemm$unstandardized$LatentClass==1,3]

################################
# 1. variance of eta
################################

# mixture part predictor variables:
kap.g1 <- c(kap1[1],kap2[1])                           #E(xi|C=1)
kap.g2 <- c(kap1[2],kap2[2])                           #E(xi|C=2)
phg1 <- matrix(c(ph11[1],ph12[1],ph12[1],ph22[1]),2,2) #Var(xi|C=1)
phg2 <- matrix(c(ph11[2],ph12[2],ph12[2],ph22[2]),2,2) #Var(xi|C=2)
kap    <- pi1[1]*kap.g1+pi1[2]*kap.g2                  #E(xi)

# covariance matrix of mixture components
phimix <- phimix.fun(kap.g1,kap.g2,phg1,phg2,pi1)
# variance of eta
ph00  <- b1%*%phimix%*%b1+ps1  

################################
# 2. Standardization
################################
#standardized regression coefficients
gamma.st <- c((b1[1]+b1[3]*kap[2]+2*b1[4]*kap[1])*sqrt(phimix[1,1]/ph00), #b11
              (b1[2]+b1[3]*kap[1]+2*b1[5]*kap[2])*sqrt(phimix[2,2]/ph00), #b12
              b1[3]*sqrt(phimix[1,1]*phimix[2,2]/ph00), #b13
              b1[4]*sqrt(phimix[1,1]*phimix[1,1]/ph00), #b14
              b1[5]*sqrt(phimix[2,2]*phimix[2,2]/ph00)) #b15

# proportion of unexplained variance
psi.st <- ps1/ph00
# standardized intercept
alpha.st <- -(b1[3]*phimix[2,1]+b1[4]*phimix[1,1]+b1[5]*phimix[2,2])/sqrt(ph00)

#standardized coefficients
gamma.st
alpha.st
psi.st


##################################################
# calculate expected value for dependent variable
##################################################
yhat <- function(x,y){
  # function returns expected value
  # input: x and y == two predictor variables
  # gamma.st,alpha.st: standardized coefficients
  yhat <- alpha.st+gamma.st[1]*x+gamma.st[2]*y+gamma.st[3]*x*y+gamma.st[4]*x^2+gamma.st[5]*y^2
  return(yhat)
}

#################################
# 3d plot
#################################
# limits of the predictor variables (min, max)
yr <- xr <- c(-3,3)
curve_3d <- function(yhat, 
                     x_range=xr, 
                     y_range=yr, 
                     col=1:6){
  # function produces 3d plot
  #input: function for dependent variable (yhat)
  #       ranges for predictor variables
  #       colors for graphic
  if (!require(rgl) ) {stop("load rgl")}
  #produces sequence for predictor variables
  xvec <- seq(x_range[1], x_range[2], len=50)
  yvec <- seq(y_range[1], y_range[2], len=50)
  # computes values for dependent variable
  fz   <- outer(xvec, yvec, FUN=yhat)
  open3d()
  # creates graphic with specifics such as labels
  persp3d(xvec, yvec, fz,col="white",
          xlab="Online Activities",ylab="Reading Attitude",zlab="Predicted Reading Skills",smooth=F)
}

#opens the graphic
curve_3d(yhat)
#saves the graphic as an external file
#snapshot3d("indapp.png")
#################################


#################################
# 3d densityplot for predictor variables
#################################
multnorm <- function(x,y){#x<-1;y<-1
  N <- length(x)
  d1 <- d2 <- c()
  #specifies class-specific multivariate normal distribution
  for(i in 1:N){
    d1[i] <- (2*pi)^(-1)*det(phg1)^(-1/2)*exp(-1/2*matrix((c(x[i],y[i])-kap.g1),1,2)%*%
                                                    solve(phg1)%*%matrix((c(x[i],y[i])-kap.g1),2,1))
    d2[i] <- (2*pi)^(-1)*det(phg2)^(-1/2)*exp(-1/2*matrix((c(x[i],y[i])-kap.g2),1,2)%*%
                                                    solve(phg2)%*%matrix((c(x[i],y[i])-kap.g1),2,1))
  }
  
  # specifies mixture distribution
  d <- pi1[1]*d1+pi1[2]*d2
  return(d)
}

#limits of the distribution
xr2 <- c(kap[1]-3*sqrt(phimix[1,1]),
         kap[1]+3*sqrt(phimix[1,1])) #read
yr2 <- c(kap[2]-3*sqrt(phimix[2,2]),
         kap[2]+3*sqrt(phimix[2,2])) #online

######################
# 3d curve (same as above for yhut in part II)
curve_3d2 <- function(multnorm, 
                      x_range=xr2, 
                      y_range=yr2, 
                      col=1:6){
  if (!require(rgl) ) {stop("load rgl")}
  xvec <- seq(x_range[1], x_range[2], len=50)
  yvec <- seq(y_range[1], y_range[2], len=50) 
  fz   <- outer(xvec, yvec, FUN=multnorm)
  open3d()
  persp3d(xvec, yvec, fz,col="grey",xlab="Online Activity",ylab="Reading Attitude",zlab="Density",smooth=F)
}

curve_3d2(multnorm)
snapshot3d("distribution.png")
######################
